#include <afxwin.h>                             // MFC-Definitionen einbinden
#include <math.h>                               // Sinus und Cosinus einbinden
#include "drei.h"                               // Headerfile einbinden


CDreiApp DreiApp;                               // Programm statisch erzeugen

BOOL CDreiApp::InitInstance()                   // Start berladen
{
    Enable3dControls();                         // 3D-Stil aktivieren

    m_pMainWnd = new CDisplay();                // Zeichenflche ist Hauptfenster
    m_pMainWnd->ShowWindow(SW_SHOW);            // Zeiger darauf speichern

    return TRUE;
}


BEGIN_MESSAGE_MAP(CDisplay, CFrameWnd)          // Nachrichten weiterleiten
    ON_WM_PAINT()
    ON_WM_CLOSE()
END_MESSAGE_MAP()


static CDreiDlg* pDlg;                          // Zugriff auf Dialog ermglichen

CDisplay::CDisplay()                            // Aktivierung Hauptfenster
{
    Create(NULL, "Huschelmutzs Vision", WS_OVERLAPPEDWINDOW);
                                                // Dialog nicht-modal kreieren
    pDlg = new CDreiDlg(this);                  // Zeiger sichern
}


void CDisplay::OnPaint()                        // Neuzeichnen
{
    pDlg->GetNumbers();                         // Zahlenwerte holen

    if ((pDlg->nRekursionstiefe==0)||(pDlg->nBorsten[0].Laenge==0))
        return;                                 // wenn kein Fraktal entsteht abbrechen

    CPaintDC MyDC(this);                        // Zeichenflche holen
    CPen pen;                                   // Stift holen
    pen.CreatePen(PS_SOLID,1,RGB(0,0,0));       // Stift soll schwarz sein mit Breite 1
    MyDC.SelectObject(&pen);                    // Stift einbinden
    
    pDC = &MyDC;                                // Zeichenflche fr CalcFractal sichern

    CRect rect;                                 // Bildschirmausmae holen
    GetClientRect(rect);
    MidX = rect.right/2;                        // Mitte bestimmen
    MidY = rect.bottom/2;

    OriginalLaenge = pDlg->nBorsten[0].Laenge;  // Originallnge holen

    for (int Lauf=0; Lauf<BorstenPunkte; Lauf++)
    {                                           // alle Borstenpunkte durchlaufen
        nBorsten[Lauf].Winkel = pDlg->nBorsten[Lauf].Winkel;
                                                // Winkel bernehmen
        nBorsten[Lauf].Laenge = pDlg->nBorsten[Lauf].Laenge/OriginalLaenge;
                                                // neue Lnge als Verhltnis zur
    }                                           // Originallnge

    CalcFractal(0,0,nBorsten[0].Winkel,1,1);    // Berechnung beginnen im Ursprung
                                                // bei Mutterborste, Normalgre und
                                                // Rekursionstiefe 1
}


void CDisplay::OnClose()                        // Programmende
{
    if (MessageBox("Wirklich beenden ?","Ende",MB_YESNO|MB_ICONQUESTION)!=IDYES)
        return;                                 // Sicherheitsabfrage
        
    pDlg->DestroyWindow();                      // nicht-modalen Dialog zerstren
    delete(pDlg);

    CFrameWnd::OnClose();                       // Zeichenflche schlieen
}


void CDisplay::CalcFractal(float StartX, float StartY, float Winkel, float Faktor, int nSchritt)
{                                               // Fraktal berechnen
    float NeueLaenge = Faktor*OriginalLaenge;   // Lnge bestimmen
    if (NeueLaenge<MaxAufloesung)               // Unterschreitung des Grenzwertes
        return;

    pDC->MoveTo(int(StartX)+MidX, -int(StartY)+MidY);
    pDC->LineTo(int(StartX+cos(Winkel)*NeueLaenge)+MidX, -int(StartY+sin(Winkel)*NeueLaenge)+MidY);
                                                // Linie zeichnen (mittels Turtle-Grafik)
    if (nSchritt>pDlg->nRekursionstiefe)        // Rekursionstiefe erreicht ?
        return;                                 // wenn ja, dann abbrechen

    float NewX = StartX+cos(nBorsten[1].Winkel+Winkel)*nBorsten[1].Laenge*NeueLaenge;
    float NewY = StartY+sin(nBorsten[1].Winkel+Winkel)*nBorsten[1].Laenge*NeueLaenge;
                                                // neue Mitte des Koordinatensystems
    CalcFractal(NewX,NewY,Winkel+nBorsten[2].Winkel,Faktor*nBorsten[2].Laenge,nSchritt+1);
                                                // 1.Tochterborste berechnen

    NewX = StartX+cos(nBorsten[3].Winkel+Winkel)*nBorsten[3].Laenge*NeueLaenge;
    NewY = StartY+sin(nBorsten[3].Winkel+Winkel)*nBorsten[3].Laenge*NeueLaenge;
                                                // neue Mitte des Koordinatensystems
    CalcFractal(NewX,NewY,Winkel+nBorsten[4].Winkel,Faktor*nBorsten[4].Laenge,nSchritt+1);
                                                // 2.Tochterborste berechnen
}


BEGIN_MESSAGE_MAP(CDreiDlg, CDialog)            // Nachrichten weiterleiten
    ON_BN_CLICKED(ID_BUTTON_ZEICHNEN, OnZeichnen)
    ON_BN_CLICKED(ID_BUTTON_ABOUT, OnAbout)
END_MESSAGE_MAP()


CDreiDlg::CDreiDlg(CWnd* pParent)               // Konstruktor
{
    Create(IDD, pParent);                       // nicht-modal erzeugen
}


BOOL CDreiDlg::OnInitDialog()                   // Dialogstart
{
    CDialog::OnInitDialog();                    // zur Grundklasse weiterleiten

    (GetDlgItem(ID_EDIT_MUTTER_ENDE_X))->SetFocus();
                                                // Focus setzen
    return FALSE;                               // diesen von Windows nicht verndern
}


void CDreiDlg::OnOK()                           // Eingabe von Enter
{
    OnZeichnen();                               // Fraktal neu zeichnen
}


void CDreiDlg::OnZeichnen()                     // explizite Zeichnen-Anforderung
{
    DreiApp.m_pMainWnd->Invalidate();           // OnPaint der Zeichenflche indirekt
                                                // aufrufen
}


void CDreiDlg::GetNumbers()                     // Dialog auslesen
{
    int nID = ID_EDIT_MUTTER_ENDE_X;            // erstes Eingabefeld

    for (int Lauf = 0; Lauf<BorstenPunkte; Lauf++)
    {                                           // fr jeden Borstenpunkt
        nBorsten[Lauf].Winkel = GetDlgItemInt(nID++)/180.0*3.1415926;
                                                // Winkel gleich in Radiant umrechnen
        nBorsten[Lauf].Laenge = GetDlgItemInt(nID++);
                                                // Lnge unverndert bernehmen
    }

    nRekursionstiefe = GetDlgItemInt(ID_EDIT_REKURSIONSTIEFE);
                                                // Rekursionstiefe auslesen
}


void CDreiDlg::OnAbout()                        // Programminfo
{
    MessageBox("geschrieben von Stephan Brumme\nin Watcom C++ 10.6 mit MFC 3.2",
               "ber...",MB_OK|MB_ICONINFORMATION);
}

